#include "edid.h"

//unsigned char vga_edid[256];
/*= 
    {0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00,
     0x10, 0xAC, 0x5A, 0x40, 0x53, 0x44, 0x47, 0x45,
     0x09, 0x15, 0x01, 0x03, 0x0E, 0x26, 0x1E, 0x78,
     0xEE, 0xEE, 0x95, 0xA3, 0x54, 0x4C, 0x99, 0x26,
     0x0F, 0x50, 0x54, 0xA5, 0x4B, 0x00, 0x71, 0x4F,
     0x81, 0x80, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
     0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x30, 0x2A,
     0x00, 0x98, 0x51, 0x00, 0x2A, 0x40, 0x30, 0x70,
     0x13, 0x00, 0x78, 0x2D, 0x11, 0x00, 0x00, 0x1E,
     0x00, 0x00, 0x00, 0xFF, 0x00, 0x43, 0x48, 0x52,
     0x59, 0x4B, 0x31, 0x33, 0x33, 0x45, 0x47, 0x44,
     0x53, 0x0A, 0x00, 0x00, 0x00, 0xFC, 0x00, 0x44,
     0x45, 0x4C, 0x4C, 0x20, 0x50, 0x31, 0x39, 0x30,
     0x53, 0x0A, 0x20, 0x20, 0x00, 0x00, 0x00, 0xFD,
     0x00, 0x38, 0x4C, 0x1E, 0x51, 0x0E, 0x00, 0x0A,
     0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x00, 0xBA,
     };	*/

unsigned char audio_edid[128] = 
    {0x02, 0x03, 0x15, 0x71, 0x41, 0x81, 0x23, 0x09,
    0x07, 0x07, 0x83, 0x01, 0x00, 0x00, 0x66, 0x03,
    0x0C, 0x00, 0x10, 0x00, 0x80, 0x00, 0x00, 0x00,
    0x10, 0x00, 0x0A, 0x20, 0x20, 0x20, 0x20, 0x20,
    0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x00,
    0x00, 0x00, 0x10, 0x00, 0x0A, 0x20, 0x20, 0x20,
    0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
    0x20, 0x00, 0x00, 0x00, 0x10, 0x00, 0x0A, 0x20,
    0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
    0x20, 0x20, 0x20, 0x00, 0x00, 0x00, 0x10, 0x00,
    0x0A, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
    0x20, 0x20, 0x20, 0x20, 0x20, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x88};
    
unsigned char dtd_640x480[18] = 
    { 0x8C, 0x0A, 0x80, 0xA0, 0x20, 0xE0, 0x2D, 0x10,
      0x10, 0x60, 0xA2, 0x00, 0x80, 0xE0, 0x21, 0x00,
      0x00, 0x18 };

int min_pclk_from_std_tim_info(unsigned char *bytes)
{
     int xres = 0;
     int yres = 0;
     int vsfreq = 0;
     int xratio[] = {16, 4, 5, 16};
     int yratio[] = {10, 3, 4, 9};
     int min_pclk;

     if (bytes[0]==0x01 && bytes[1]==0x01)
        return 0;
     
     xres = (bytes[0] + 31) * 8;
     yres = (xres * yratio[(bytes[1] & 0xC0) >> 6]) / xratio[(bytes[1] & 0xC0) >> 6];
     vsfreq = (bytes[1] & 0x3F) + 60;
     
     min_pclk = ((xres+1) * (yres+1) * vsfreq) / 1000000; // for PCLK in MHz 
     return min_pclk;
}

unsigned char min(unsigned char a, unsigned char b)
{
     return a < b ? a : b;
}

unsigned char calculate_checksum(unsigned char *block_beg)
{
     unsigned char sum=0;
     unsigned char i;
     
     for (i=0; i<0x7F; i++)
         sum += block_beg[i];
         
     return (~sum)+1; // Check this?!
}

void edid_prepare(unsigned char *new_edid)
{
     unsigned char i, j;
     unsigned char *ptr;

     // 0:19 = Header information
     // 19:24 = Basic display information
     // 25:34 = Chromacity coords
     // 35:37 = Established bitmap timings (All these VESA can be supported
     
     new_edid[20] = 0; // Setting Video Input Parameters to 0 = Digital Input
     
     /* Bytes 38...53 = STD Timing Info */
     for (i=38; i<54; i+=2)
     {
         /* If minimum pclk calculated from std_timing info is above 165 MHz - 
            - the mode is dropped */
         if (min_pclk_from_std_tim_info(new_edid+i) > 165) {
                  new_edid[i]   = 0x01;
                  new_edid[i+1] = 0x01;
         }
     }
     
     /* Bytes 54...71 = Descriptor 1
        Bytes 72...89 = Descriptor 2
        Bytes 90...107 = Descriptor 3
        Bytes 108...125 = Descriptor 4 */
     for (i=54; i<125; i+=18) {
         ptr = &new_edid[i];
         
         /* Detailed Timing Information */
           if (((ptr[0]*256 + ptr[1])/100) > 165) {
              /* DTD Lists mode that is with PCLK > 165 MHz */
               for (j=0; j<18; j++)
                   ptr[j] = dtd_640x480[j]; // replacing DTD with 640x480
           } else {
               if ((ptr[0]==0 && ptr[1] == 0) && (ptr[3]==253)) {
                     // Monitor Range Descriptor      
                     ptr[9] = min(ptr[9], 17); // Minumum of: monitor's max pclk,165MHz
               }
           }
     }
     
     new_edid[126] = 1; // One CEA 861 Extension
     new_edid[127] = calculate_checksum(new_edid);
     for (i=0; i<128; i++)
          new_edid[i+0x80] = audio_edid[i];
     new_edid[255] = calculate_checksum(new_edid+0x80);
}
