/*
	This bit-bangling I2C has been taken from Wikipedia and modified
	for ADuC7000. See www.wikipedia.org/wiki/I2c/ for original source code.

		P1.2 = MCU_SCL [SCL1]
		P1.3 = MCU_SDA [SDA1]
*/

/* Hardware-Specific Support Functions That MUST Be Customized */
#define I2CSPEED  17
#include "includes.h"
#include <stdio.h>
#include "i2c_bb.h"

void I2CDELAY(void) { int i; for(i=0;i<I2CSPEED;i++); }
bool READSCL(void)
{   
	unsigned char i=0;

	GP1DAT = GP1DAT & 0xFBFBFFFF; // unsetting direction, unsetting output;
	for (i=0; i<2;i++);
	return (GP1DAT & 0x04) >> 2;
}  /* Set SCL as input and return current level of line, 0 or 1 */

bool READSDA(void)
{
	unsigned char i=0;

	GP1DAT = GP1DAT & 0xF7F7FFFF; // unsetting direction, unsetting output;
	for (i=0; i<2;i++);

	return (GP1DAT & 0x08) >> 3;
}  /* Set SDA as input and return current level of line, 0 or 1 */

void CLRSCL(void)
{
	GP1DAT = (GP1DAT | 0x04000000) & 0xFFFBFFFF; // sets as an output and set low;;
} /* Actively drive SCL signal low */

void CLRSDA(void) {
	GP1DAT = (GP1DAT | 0x08000000) & 0xFFF7FFFF; // sets as an output and set low;;
} /* Actively drive SDA signal low */

void ARBITRATION_LOST(void) {}
 
/* Global Data */
bool started = false;
 
void i2c_start_cond(void)
{
        /* if started, do a restart cond */
        if (started) {
                /* set SDA to 1 */
                READSDA();
                I2CDELAY();
                /* Clock stretching */
                while (READSCL() == 0)
                        ;  /* You should add timeout to this loop */
        }
        if (READSDA() == 0)
                ARBITRATION_LOST();
        /* SCL is high, set SDA from 1 to 0 */
        CLRSDA();
        I2CDELAY();
        CLRSCL();
        started = true;
}
 
void i2c_stop_cond(void)
{
        /* set SDA to 0 */
        CLRSDA();
        I2CDELAY();
        /* Clock stretching */
        while (READSCL() == 0)
                ;  /* You should add timeout to this loop */
        /* SCL is high, set SDA from 0 to 1 */
        if (READSDA() == 0)
                ARBITRATION_LOST();
        I2CDELAY();
        started = false;
}
 
/* Write a bit to I2C bus */
void i2c_write_bit(bool bit)
{
        if (bit) 
                READSDA();
        else 
                CLRSDA();
        I2CDELAY();
        /* Clock stretching */
        while (READSCL() == 0)
                ;  /* You should add timeout to this loop */
        /* SCL is high, now data is valid */
        /* If SDA is high, check that nobody else is driving SDA */
        if (bit && READSDA() == 0) 
                ARBITRATION_LOST();
        I2CDELAY();
        CLRSCL();
}
 
/* Read a bit from I2C bus */
bool i2c_read_bit(void)
{
        bool bit;
        /* Let the slave drive data */
        READSDA();
        I2CDELAY();
        /* Clock stretching */
        while (READSCL() == 0)
                ;  /* You should add timeout to this loop */
        /* SCL is high, now data is valid */
        bit = READSDA();
        I2CDELAY();
        CLRSCL();
        return bit;
}
 
/* Write a byte to I2C bus. Return 0 if ack by the slave */
bool i2c_write_byte(bool send_start, bool send_stop, unsigned char byte)
{
        unsigned bit;
        bool nack;
        if (send_start) 
                i2c_start_cond();
        for (bit = 0; bit < 8; bit++) {
                i2c_write_bit((byte & 0x80) != 0);
                byte <<= 1;
        }
        nack = i2c_read_bit();
        if (send_stop)
                i2c_stop_cond();
        return nack;
}
 
/* Read a byte from I2C bus */
unsigned char i2c_read_byte(bool nack, bool send_stop)
{
        unsigned char byte = 0;
        unsigned bit;
        for (bit = 0; bit < 8; bit++)
                byte = (byte << 1) | i2c_read_bit();             
        i2c_write_bit(nack);
        if (send_stop)
                i2c_stop_cond();
        return byte;
}

void i2c_del(void)
{
   unsigned int i;
   for (i=0; i<334; i++);
}

void i2c_sdel(void)
{
   unsigned int i;
   for (i=0; i<50; i++);
}

void i2cReadReg8bb(unsigned char device_addr, unsigned char sub_addr, unsigned char *data_buffer)
{
//   unsigned char nack = 0;
//   nack = 
   i2c_write_byte(1, 0, device_addr); // SEND START, NO STOP, ADDR
/*   if (!(nack)) {
       printf("I2C Device (%02X) doesn't exist;", device_addr);
	   return;
   }*/
   i2c_del();
   i2c_write_byte(0, 0, sub_addr); // START, STOP, SUBADDR
   i2c_del();
   i2c_write_byte(1, 0, device_addr+1); // START, NO STOP, SUBADDRESS
   i2c_del();
   *data_buffer = i2c_read_byte(1, 1);
}

void i2cWriteReg8bb(unsigned char device_addr, unsigned char sub_addr, unsigned char value)
{
   //unsigned char nack = 0;
   //nack =
   i2c_write_byte(1, 0, device_addr); // SEND START, NO STOP, ADDR
   i2c_del();
   i2c_write_byte(0, 0, sub_addr); // START, STOP, SUBADDR
   i2c_del();
   i2c_write_byte(0, 1, value); // START, NO STOP, SUBADDRESS
   i2c_del();
}


