/*
	CFTL Analog Devices 2012.
	Author: Witold Kaczurba
	Rev. 0.1 11th of June 2012
*/

#include <stdio.h>
#include "includes.h"
#include "edid.h"
#include "i2c_bb.h"
#include "misc.h"
#include <string.h>

#define CLOCK 		41779200
#define CORECLOCK  	20889600
#define PRESCALER	256							 // T0 prescaler
#define T0_PERIOD   (CORECLOCK / 2) / PRESCALER  // 0.5 sec delay; it is only 16-bit counter!

unsigned char script[] = {
	0x98, 0xF4, 0x80,
	0x98, 0xF5, 0x7C,
	0x98, 0xF8, 0x4C,
	0x98, 0xF9, 0x64,
	0x98, 0xFA, 0x6C,
	0x98, 0xFB, 0x68,
	0x98, 0xFD, 0x44,
	0x98, 0x01, 0x06,
	0x98, 0x02, 0xF7,
	0x98, 0x03, 0x40,
	0x98, 0x04, 0x42,
	0x98, 0x05, 0x28,
	0x98, 0x06, 0xA6,
	0x98, 0x0B, 0x44,
	0x98, 0x0C, 0x42,
	0x98, 0x15, 0x80,
	0x98, 0x19, 0x8A,
	0x98, 0x14, 0x7F,
	0x98, 0x33, 0x40,
	0x44, 0xBA, 0x01,
	0x64, 0x40, 0x81,
	0x68, 0x9B, 0x03,
	0x68, 0xC1, 0x01,
	0x68, 0xC2, 0x01,
	0x68, 0xC3, 0x01,
	0x68, 0xC4, 0x01,
	0x68, 0xC5, 0x01,
	0x68, 0xC6, 0x01,
	0x68, 0xC7, 0x01,
	0x68, 0xC8, 0x01,
	0x68, 0xC9, 0x01,
	0x68, 0xCA, 0x01,
	0x68, 0xCB, 0x01,
	0x68, 0xCC, 0x01,
	0x68, 0x00, 0x00,
	0x68, 0x83, 0xFE,
	0x68, 0x6F, 0x08,
	0x68, 0x85, 0x1F,
	0x68, 0x87, 0x70,
	0x68, 0x8D, 0x04,
	0x68, 0x8E, 0x1E,
	0x68, 0x1A, 0x8A,
	0x68, 0x57, 0xDA,
	0x68, 0x58, 0x01,
	0x68, 0x75, 0x10,
	0x34, 0x00, 0x80,
	0x34, 0x02, 0x80,
	0x34, 0x08, 0x10,
	0x34, 0x0A, 0x06,
	0x34, 0x0C, 0x17,
	0x34, 0x0E, 0x0A,
	0x34, 0x12, 0x01,
	0xFF };

void I2C_write_tab_new( unsigned char *script )
{
	do {
		i2cWriteReg8bb(*script, *(script+1), *(script+2));
		script += 3;
	} while (*script != 0xFF)  ;
}

void i2c_vga_edid_read(unsigned char *ptr)
{
    unsigned int addr=0;
	unsigned char i=0;

	for (addr=0; addr<256; addr++)
		i2cReadReg8bb(0xA0, addr, (ptr+addr));

	/* output EDID table */
	printf("\r\nEDID Content:");
	printf("\r\n  | 00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F");
	printf("\r\n--| -----------------------------------------------");
	for (addr=0; addr<256; addr++) {
		if (addr % 16 == 0)
			printf("\r\n%02X| ", addr / 16);
		printf("%02X ", *(ptr+addr));
	   	for (i=0; i<200; i++);
	}
	printf("\r\n");
}

void i2c_program_edid(unsigned char *data, int length)
{
	unsigned long i;
	i2cWriteReg8bb(0x68, 0x6C, 0xA3); // enable manual HPA
	i2cWriteReg8bb(0x98, 0x20, 0x70); // HPD low
	i2cWriteReg8bb(0x64, 0x74, 0x00); // disable internal EDID

	for (i=0; i<length; i++)
		i2cWriteReg8bb(0x6C, i, data[i]);

	for (i=0; i<800000; i++); // delay for HPA

	i2cWriteReg8bb(0x64, 0x74, 0x01); // enable internal EDID
	i2cWriteReg8bb(0x98, 0x20, 0xF0); // HPD high
	i2cWriteReg8bb(0x68, 0x6C, 0xA3); // disable manual HPA
}

int main(void)
{
	unsigned char vga_edid[256];

	/* Pin configuration:
		P0.0 = Serial Download button 1k to GND
		P0.3 = floating
		P0.4 = floating
		P0.5 = ADV761X_INT1
		P0.6 xxxx
		P0.7 = floating
		P1.0 = UART [SIN]
		P1.1 = UART [SOUT]
		P1.2 = MCU_SCL [SCL1]
		P1.3 = MCU_SDA [SDA1]
		P1.4 = ADV761X_INT2 [GPIO/IRQ2]
		P1.5 = V_RESET (VCC_3V3 pullup 4k7) [GPIO/IRQ3]
		P1.6 = DAC_POWER [GPIO]
		P1.7 = floating [GPIO]
		P2.0 = I2C_SWITCH: hgih = VGA_EDID; low = MAIN I2C (or the other way :))
		P4.2 = LED0
	*/

	GP0CON = 0x00000000; // all set to GPIO
    GP0DAT = 0x0000F9F9; // all zeroed input pins 
    GP1CON = 0x00000011; // Configure I2C lines SCL1 and SDA1
	GP1DAT = 0x0C0C00FF; // SCL1, SDA1 output
    GP2CON = 0x00000000; // ALL set to GPIO; GPIO2 = I2C_SWITCH (P2.0)  */
	GP2DAT = 0x01000000; // Setting GP2 P2.0 = LOW
	GP4DAT = 0x04000000; // GPIO for LED0 (P4.2)

	/* PLLs programming */
	PLLKEY1 = 0xAA;
	PLLCON = 0x21;
	PLLKEY2 = 0x55;

	POWKEY1 = 0x01;
	POWCON = 0x00;
	POWKEY2 = 0xF4;

    ConfigUARTSpeed9600();

    i2c_to_edid(); // switch I2C to EDID
	gpio_led_on();	// ADV7619
	i2c_vga_edid_read(vga_edid); // vga_edid
	edid_prepare(vga_edid); // Modifies EDID read from monitor and adds CEA861 block.

	gpio_led_off();	// VGA EDID
	i2c_to_hdmirx(); // switch to HDMI RX
	i2cWriteReg8bb(0x98, 0xFF, 0x80); // RESET part and wait (no ACK after reset)
	wait2();

	I2C_write_tab_new(script); // program the part
	i2c_program_edid(vga_edid, 256); // program modified EDID

	while (1);
}
